# jwts
A package for Signing/Creating and Verifying JWTs, with build in asymmetric key pair generation for the token signature with the crypto module.  
[NPM package link](https://www.npmjs.com/package/jwts)  
Note that this package is in the beta development phase. Any issues, please open on Github and contributions or comments are all welcomed too. Thanks for giving this a try!


## Installation
```cli
# Install into node_modules/ and save as dependency in package.json
npm i jwts

# Install package to use in production without installing the package's devDependencies
npm i jwts --production
```
This package, when installed using npm, contains only the required source files without all the tests and example codes.  
To view the full codebase, clone this repository from [Github](https://github.com/Jaimeloeuf/jwts) instead.


## How to use
- For examples on how to use this package, refer to the modules in the "example_code" directory. It includes a comprehensive list of all use cases for this package. Download the repo from the Github page, to access this directory.
- To see how you can use this package to build a full Identity and Access Management service, refer to [this](https://github.com/Jaimeloeuf/police-man) IAM microservice for example.
###### Basic usecase (Note that all import statements are sync/blocking):
```js
// Require jwts to use the methods without using automated Key generation and binding
const jwt = require('jwts');

/* Assuming you got the publicKey from a secret management service. Create
   a verification function by partially applying in the publicKey and the
   default verification options.	*/
const verify = jwt.verify_token(publicKey)({
	issuer: "auth-backend",
    audience: "my_service"	// Enter your default verify token options
})


/*  Assuming the "jwt" is the jwt you want to verify
    "verify" will return a Promise, that will resolve with the verified and decoded token,
    else it will reject with an error if the verification failed.   */
verify(jwt);

// Assuming you want to verify the jwt, with a different set of options
verify(jwt, {
	audience: "admin_service" // Optional options to override your default options
});
```
```js
/* Directly call the "apply_keys" method if you want to both create and verify the
   tokens with automatically generated RSA key pair binded into the functions.	*/
const jwt = require('jwts').apply_keys();

jwt.getPublicKey(); // Get the generated publicKey

/* Apply default options object into the create and verify functions */
const create = jwt.create_token({
	issuer: "auth-backend",
    audience: "my_service"	// Enter your default verify token options
});
const verify = jwt.verify_token({
	issuer: "auth-backend",
    audience: "my_service"	// Enter your default verify token options
});


// Self invoked async function to use await on the Promises
(async function() {
    /*  "create" returns a Promise,
        that will resolve with the signed and encoded token,
        else it will reject with an error.  */
    const token = await create({
        user: "james",
        roles: "admin"
    });
    
    /*  "verify" returns a Promise, that will resolve with the verified and decoded token,
        else it will reject with an error if the verification failed.   */
    const decoded_token = await verify(token);
})()
```


## JWTs, general structure and F.A.Qs
##### F.A.Q
- What should a JWT contain?   (Client holding the JWT will be referred to as the owner)
  - The token should be self-identifying of its use, e.g. identity token / information token...
  - The owner's Identity, basically declaring who the user is
  - What are the resources that the owner can access.
  - Who issued the JWT to the user
  - Who is the JWT intended for? Meaning which microservices should accept the token?
  - The access right of the owner. (What are the resources he or she is allowed to access)
  - JWTs should contain all data needed to fulfill the request and nothing more.
  - Do not put unnecessary claims into the JWT such as user data. (Keep these in the DB)
- What are JWTs used for?
  - JWTs are used for stateless Authentication
  - JWTs are used to prove access rights of a bearer
  - JWTs facilitate trusted data transfer in a stateless request
##### Example on what a JWT contains
From here on out, the client browser who holds the JWT, or a service or whatever that holds the JWT, will be referred to as the "bearer".

^ Make the above important stuff be bold and/or italized


In a JWT, especially one used as identity validator, it is suggested that you include the following key:value pairs:
- "sub", short for subject
    - This value should allow the service to identitfy Who this person is, or who is the bearer of this identity token
- "scope"  /  "roles"
    - Include a key:value pair that stores an array as the value
    - The array should allow the service accepting the tokens to determine What this person can access with this token
    - This should include things like read/write permissions to data in the database, service / application specific actions such as for example "account creation".
    - The scope should be used for laying out the permissions, and the roles should define the the !!!!Group Access Policies!!!!!!
        - Basically access management grouped by what type of user is this.
        - So there should be another part of the service that specifically defines all user groups, and the actions that they are allowed to do.
- When the token expires (exp)
- Who issued the token (iss, short for issuer)
```js
    {
    	// Token headers
        "typ": "JWT",
        "alg": "HS256" // The algorithm used for the signature is HMAC SHA-256
    }
    {
        
        // These below declarations are known as Claims, because the token creator claims a set of assertions that can be used to ‘know’ things about the subject. Because the token is signed with a secret key, you can verify its signature and implicitly trust what is claimed.
        "exp": ,
        "iat": ,
        "expiresIn": ,
        "tokenType": "Bearer",
        "sub":
        "subject": "retrieve data", // What is the purpose of this token/request?
        "usrID": 578ec9,
        "usr": "john@gmail.com",
        "iss": "bookings.com", // Issuer of the token
        "aud": "bookings.com/", // Intended audience that should acccept the token
        "account type": "consumer", // The type of account that the user has
        "roles": {
            // The things/roles that the user is allowed to do
            "role": "consumer"
            "booking": "create"
        }
        "scope": ["read", "write", "update", "del"]
    }
```


## Dependencies
- If token signing feature is to be used, then the native [crypto](https://nodejs.org/api/crypto.html) module must be available for automatic RSA key generation and binding.
- Note that if the native crypto module is not available, the process will be forced to quit!
- This package is built on top of the [jsonwebtoken](https://www.npmjs.com/package/jsonwebtoken) package, thus the dependencies includes the jsonwebtoken package and its dependencies.


## Author
2019 Jaime Loeuf


## License
This package is made under the MIT license.


## Todo

Why is the aircon in the train/bus/buildings so cold? Can the government do anything about it with thins like bills and laws?
What are the current protection that singapores has on the environment? Like a limit of land use or whatever?
Call the state department or whatever and say that I am an independant journalist doing some research.
Write an article on all the things singapore is currently doing about climate change. < should be at least a 10min read article.
What is the truth behind the country that claims to be a green garden city?

- Write a git hook to auto publish to NPM when pushing a new commit that is tagged with a release version
- For DoVue create the line by line parser that will store a note as an array of lines. Which can be individually changed and commited to the remote server

Add in a self timed thing,
where the key is regen every few seconds/min
Package this to have a dist/ folder. Where the code is bundled and minified!
^ Should this be done? Because if so, there will be no comments like JS Docs which will lead to blind mouse situation in a IDE
^ Perhaps a better way would be to inform the user on how to get the bundled version of this package.




If token signing feature is to be used, then the native crypto module must be available for automatic RSA key generation and binding.
Note that if the native crypto module is not available, the process will be forced to quit!
^ This above in the README is bad.
Make it so that the process only quits if the user have no crypto, and is trying to do automatic key gen. If there is no crypto but user not using auto key gen, then yes, it is fine for the user to continue on.


https://www.npmjs.com/package/jwt-generator
^ Look over this to see what can be learned from this.